package de.hsbo.fbv.bmg.topology.networks.impl;

import de.hsbo.fbv.bmg.topology.networks.Element;

/**
 * 
 * @author zimmermann
 *
 * @param <ElemT>
 */
public class SimpleElement<ElemT> implements Element<ElemT>, Comparable {
	
	// Parameter
	public static final int OBJECT = 1;
	public static final int TRACE  = 2;
	public static final int WEIGHT = 4;
	public static final int STATE =  8;
	
	static int printOption = 0;
	static int sortOption = 0;
	
	// Attribute
	ElemT object = null;		// Bezugsobjekt
	Element trace;				// Vorlufer
	double weight = 0.0;		// Knoten- bzw. Kantengewicht
	int state = 0;				// Zustand
	
	
	SimpleElement(ElemT object) {
		super();
		this.object = object;
	}
	

	@Override
	public ElemT getObject() {
		return object;
	}

	@Override
	public double getWeight() {
		return weight;
	}

	@Override
	public int getState() {
		return state;
	}

	@Override
	public Element getTrace() {
		return trace;
	}

	public void setWeight(double weight) {
		this.weight = weight;
	}

	public void setState(int state) {
		this.state = state;
	}

	public void setTrace(Element trace) {
		this.trace = trace;
	}

	@Override
	public String toString() {
		if(printOption==0)
			return "object=" + object + ", trace=" + trace
				+ ", weight=" + weight + ", state=" + state;
		else {
			String s = "";
			if((printOption & OBJECT) != 0)
				s = s + object +", ";
			if((printOption & TRACE) != 0)
				s = s + trace +", ";
			if((printOption & WEIGHT) != 0)
				s = s + weight +", ";
			if((printOption & STATE) != 0)
				s = s + state;
			if(s.endsWith(", "))
				s = s.substring(0, s.length()-2);
			return s;
		}
	}

	public static void setPrintoptions(int printOption) {
		SimpleElement.printOption = printOption;
	}

	@Override
	public int compareTo(Object o) {
		Element eo = (Element)o;
		if(this.sortOption==0)
			return 0;
		else if (sortOption==OBJECT) {
			Comparable ct = (Comparable)this.object;
			Comparable ce = (Comparable)(eo.getObject());
			return ct.compareTo(ce);
		}
		else if(sortOption==TRACE) {
			Integer ht = (Integer)this.hashCode();
			Integer he = (Integer)eo.hashCode();
			return ht.compareTo(he);
		}
		else if(sortOption==WEIGHT) {
			Double dt = (Double)this.weight;
			Double de = (Double)eo.getWeight();
			return dt.compareTo(de);
		}
		else if(sortOption==STATE) {
			Integer st = (Integer)this.state;
			Integer se = (Integer)eo.getState();
			return st.compareTo(se);
		}
		else 
			return 0;
	}

	public static void setSortOption(int sortOption) {
		SimpleElement.sortOption = sortOption;
	}
	
	public void setValues(Element element) {
		this.weight = element.getWeight();
		this.trace = element.getTrace();
		this.state = element.getState();
	}

	
}
