package de.hsbo.fbv.bmg.topology.networks.impl;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Path2D;
import java.awt.geom.Rectangle2D;

import de.hsbo.fbv.bmg.graphics.viewer.simple.Layer;
import de.hsbo.fbv.bmg.graphics.viewer.simple.StyledShapes;
import de.hsbo.fbv.bmg.topology.networks.Edge;
import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.networks.Node;


/**
 * NetworkLayer - Adapterklasse von Layer, die Knoten- und Kantenobjekte speichern kann.
 * Die Knoten- und Kantenobjekte mssen vom Datentyp des Interface Shape sein. 
 * @author Zimmermann, HS Bochum 2011<br>
 *
 * @param <NodeT> Knotentyp, Adapterklasse von Shape
 * @param <EdgeT> Kantentyp, Adapterklasse von Shape
 * 
 * @see Layer
 * @see Network1T
 */
public class SimpleNetworkLayer<NodeT extends Shape, EdgeT extends Shape> extends SimpleNetwork<NodeT, EdgeT> implements Layer {

	// Attribute
	boolean visible = true;
	Color nodeColor = Color.BLACK;
	Color edgeColor = Color.BLACK;
	Stroke nodeStroke = new BasicStroke(3.f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
	Stroke edgeStroke = new BasicStroke();
	
	// Konstruktoren
	public SimpleNetworkLayer() {
		super();
	}
	
	public SimpleNetworkLayer(Network1T<NodeT, EdgeT> network) {
		super(network);
	}
	
	@Override
	public Rectangle2D getBounds2D() {
		Path2D path = new Path2D.Double();
		if(!this.isEmpty()) {
			Node<NodeT>[] nodes = this.getNodes();
			for(int i=0; i<nodes.length; i++) {
				path.append(nodes[i].getObject(), false);
			}
			Edge<EdgeT>[] edges = this.getEdges();
			for(int i=0; i<edges.length; i++) {
				path.append(edges[i].getObject(), false);
			}
			return path.getBounds2D();
		}
		else
			return null;
	}

	@Override
	public StyledShapes[] getStyledShapes() {
		StyledShapes edgeShapes = null;
		if (this.getEdges().length!=0) {
			Shape[] edgeArray = new Shape[this.getNumEdges()];
			Edge<EdgeT>[] edges = super.getEdges();
			for(int i=0; i<edges.length; i++)
				edgeArray[i] = edges[i].getObject();
			edgeShapes = new StyledShapes(edgeArray);
			edgeShapes.setDraw();
			edgeShapes.setColor(edgeColor);
			edgeShapes.setStroke(edgeStroke);
		}
		StyledShapes nodeShapes = null;
		if (this.getNodes().length!=0) {
			Shape[] nodeArray = new Shape[this.getNumNodes()];
			Node<NodeT>[] nodes = super.getNodes();
			for(int i=0; i<nodes.length; i++)
				nodeArray[i] = nodes[i].getObject();
			nodeShapes = new StyledShapes(nodeArray);
			nodeShapes.setDraw();
			nodeShapes.setColor(nodeColor);
			nodeShapes.setStroke(nodeStroke);
		}
		if(edgeShapes!=null && nodeShapes!=null)
			return new StyledShapes[] {edgeShapes, nodeShapes};
		else if(edgeShapes==null)
			return new StyledShapes[] {nodeShapes};
		else if(nodeShapes==null)
			return new StyledShapes[] {edgeShapes};
		else
			return null;
	}

	@Override
	public boolean isVisible() {
		// TODO Auto-generated method stub
		return visible;
	}

	public Color getNodeColor() {
		return nodeColor;
	}

	public void setNodeColor(Color nodeColor) {
		this.nodeColor = nodeColor;
	}

	public Color getEdgeColor() {
		return edgeColor;
	}

	public void setEdgeColor(Color edgeColor) {
		this.edgeColor = edgeColor;
	}

	public Stroke getNodeStroke() {
		return nodeStroke;
	}

	public void setNodeStroke(Stroke nodeStroke) {
		this.nodeStroke = nodeStroke;
	}

	public Stroke getEdgeStroke() {
		return edgeStroke;
	}

	public void setEdgeStroke(Stroke edgeStroke) {
		this.edgeStroke = edgeStroke;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}
	
}
